/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef sauce_h
#define sauce_h

#define SAUCE_SIZE 128

#define SAUCE_BUFFER_SIZE (255*64+SAUCE_SIZE)

typedef enum{
    DT_None = 0,
    DT_Character = 1,
    DT_Bitmap = 2,
    DT_Vector = 3,
    DT_Audio = 4,
    DT_BinaryText = 5,
    DT_XBin = 6,
    DT_Archive = 7,
    DT_Executable = 8
}sauce_DataType_t;

typedef enum{
    FT_ASCII = 0,
    FT_ANSi = 1,
    FT_ANSiMation = 2,
    FT_RIP_script = 3,
    FT_PCBoard = 4,
    FT_Avatar = 5,
    FT_HTML = 6,
    FT_Source = 7,
    FT_TundraDraw = 8
}sauce_FileType_Character_t;

typedef enum{
    FT_GIF = 0,
    FT_PCX = 1,
    FT_LBM_IFF = 2,
    FT_TGA = 3,
    FT_FLI = 4,
    FT_FLC = 5,
    FT_BMP = 6,
    FT_GL = 7,
    FT_DL = 8,
    FT_WPG = 9,
    FT_PNG = 10,
    FT_JPG_JPeg = 11,
    FT_MPG = 12,
    FT_AVI = 13
}sauce_FileType_Bitmap_t;

typedef enum{
    FT_DXF = 0,
    FT_DWG = 1,
    FT_WPG_Vector = 2,
    FT_3DS = 3
}sauce_FileType_Vector_t;

typedef enum{
    FT_MOD = 0,
    FT_669 = 1,
    FT_STM = 2,
    FT_S3M = 3,
    FT_MTM = 4,
    FT_FAR = 5,
    FT_ULT = 6,
    FT_AMF = 7,
    FT_DMF = 8,
    FT_OKT = 9,
    FT_ROL = 10,
    FT_CMF = 11,
    FT_MID = 12,
    FT_SADT = 13,
    FT_VOC = 14,
    FT_WAV = 15,
    FT_SMP8 = 16,
    FT_SMP8S = 17,
    FT_SMP16 = 18,
    FT_SMP16S = 19,
    FT_PATCH8 = 20,
    FT_PATCH16 = 21,
    FT_XM = 22,
    FT_HSC = 23,
    FT_IT = 24,
}sauce_FileType_Audio_t;

typedef enum{
    FT_ZIP = 0,
    FT_ARJ = 1,
    FT_LZH = 2,
    FT_ARC = 3,
    FT_TAR = 4,
    FT_ZOO = 5,
    FT_RAR = 6,
    FT_UC2 = 7,
    FT_PAK = 8,
    FT_SQZ = 9
}sauce_FileType_Archive_t;


typedef struct{
    size_t FileSize;
    sauce_DataType_t DataType;
    union{
        int FileType;
        sauce_FileType_Character_t FileType_Character;
        sauce_FileType_Bitmap_t FileType_Bitmap;
        sauce_FileType_Vector_t FileType_Vector;
        sauce_FileType_Audio_t FileType_Audio;
        sauce_FileType_Archive_t FileType_Archive;
    };
    int TInfo[4];
    int TFlags;
    int total_size;
}sauce_t;

#define sauce_BinaryText_width(s) ((s)->FileType>1 ? (s)->FileType*2 : sauce_width(s)*2)

#define sauce_width(s) ((s)->TInfo[0])
#define sauce_height(s) ((s)->TInfo[0])

#define sauce_ANSiFlags_iCE_Color(s) ((s)->TFlags&0x01)
#define sauce_ANSiFlags_Letter_Spacing(s) (0x02==((s)->TFlags&0x06)?8:9)
#define sauce_ANSiFlags_Aspect_Ratio(s) (0x10==((s)->TFlags&0x18)?1:0)

typedef struct{
    unsigned char buffer[SAUCE_BUFFER_SIZE];
    size_t size;
}sauce_buffer_t;

#define sauce_buffer_init(b) (b)->size=0
#define sauce_buffer_add(b,ch) if((b)->size<SAUCE_BUFFER_SIZE)(b)->buffer[(b)->size++]=(ch)
sauce_t *sauce_validate(const char*,size_t,sauce_t*);
#define sauce_buffer_validate(a,b) sauce_validate((a)->buffer,(a)->size,b)

#endif /* sauce_h */
