#ifndef doc_image_h
#define doc_image_h

#include "color.h"
#include "gfx/clip.h"
#include "image_buffer.h"
#include "pixel_format.h"

namespace doc{
    class Image{
        PixelFormat _format;
        int _width;
        int _height;
    protected:
        Image(PixelFormat f, int w, int h):_format(f),_width(w),_height(h){}
    public:
        static Image *create(PixelFormat,int width,int height,const ImageBufferPtr &buffer = ImageBufferPtr());

        PixelFormat pixelFormat() const { return _format; }
        int width() const { return _width; }
        int height() const { return _height; }

        virtual uint8_t *getPixelAddress(int x, int y) const = 0;
        virtual color_t getPixel(int x,int y) const = 0;
        virtual void putPixel(int x, int y, color_t color) = 0;
        virtual void clear(color_t) = 0;
        virtual void copy(const Image*, gfx::Clip) = 0;
        virtual void drawHLine(int x1, int y, int x2, color_t) = 0;
        virtual void fillRect(int x1, int y1, int x2, int y2, color_t) = 0;
        virtual void blendRect(int x1, int y1, int x2, int y2, color_t, int opacity) = 0;
    };
}

#endif /* doc_image_h */
