/*
 * Copyright (C) 2004 - 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <endian.h>

#include "atarist.h"

void
atarist_decode_palette(uint16_t *atari_palette, atarist_cairo_color_t *cairo_palette)
{
    int i;
    for(i=0;i<16;++i){
        uint16_t col = be16toh(atari_palette[i]);
        int r = (col>>8)&0x07;
        int g = (col>>4)&0x07;
        int b = (col>>0)&0x07;
#ifdef ATARIST_CAIRO_BITS16
        r = (r<<2) | (r>>1);
        g = (g<<3) | (g<<0);
        b = (b<<2) | (b>>1);
        cairo_palette[i] = (r<<11) | (g<<5) | (b<<0);
#else
        r = (r<<5) | (r<<2) | (r>>1);
        g = (g<<5) | (g<<2) | (g>>1);
        b = (b<<5) | (b<<2) | (b>>1);
        cairo_palette[i] = (r<<16) | (g<<8) | b;
#endif
    }
}

cairo_surface_t *
atarist_decode_image_0(uint16_t *atari_image, atarist_cairo_color_t *cairo_palette)
{
    cairo_surface_t *surface;
    atarist_cairo_color_t* dst;
    int rowskip;
    int x;
    int y;
    int i;
#ifdef ATARIST_CAIRO_BITS16
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB16_565,320,200);
#else
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,320,200);
#endif
    dst = (atarist_cairo_color_t*)cairo_image_surface_get_data(surface);
    rowskip = cairo_image_surface_get_stride(surface)/sizeof(atarist_cairo_color_t)-320;
    for(y=0;y<200;++y){
        for(x=0;x<20;++x){
            int plane1 = be16toh(atari_image[y*80+x*4+0]);
            int plane2 = be16toh(atari_image[y*80+x*4+1]);
            int plane3 = be16toh(atari_image[y*80+x*4+2]);
            int plane4 = be16toh(atari_image[y*80+x*4+3]);
            for(i=15;i>=0;--i){
                int pixel = (plane1 & (1<<i) ? 0x01 : 0) |
                            (plane2 & (1<<i) ? 0x02 : 0) |
                            (plane3 & (1<<i) ? 0x04 : 0) |
                            (plane4 & (1<<i) ? 0x08 : 0);
                *dst++ = cairo_palette[pixel];
            }
        }
        dst += rowskip;
    }
    cairo_surface_mark_dirty(surface);
    return surface;
}


cairo_surface_t *
atarist_decode_image_1(uint16_t *atari_image, atarist_cairo_color_t *cairo_palette)
{
    cairo_surface_t *surface;
    atarist_cairo_color_t* dst;
    int rowskip;
    int x;
    int y;
    int i;
#ifdef ATARIST_CAIRO_BITS16
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB16_565,640,200);
#else
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,640,200);
#endif
    dst = (atarist_cairo_color_t*)cairo_image_surface_get_data(surface);
    rowskip = cairo_image_surface_get_stride(surface)/sizeof(atarist_cairo_color_t)-640;
    for(y=0;y<200;++y){
        for(x=0;x<40;++x){
            int plane1 = be16toh(atari_image[y*80+x*2+0]);
            int plane2 = be16toh(atari_image[y*80+x*2+1]);
            for(i=15;i>=0;--i){
                int pixel = (plane1 & (1<<i) ? 0x01 : 0) |
                            (plane2 & (1<<i) ? 0x02 : 0);
                *dst++ = cairo_palette[pixel];
            }
        }
        dst += rowskip;
    }
    cairo_surface_mark_dirty(surface);
    return surface;
}


cairo_surface_t *
atarist_decode_image_2(uint16_t *atari_image, atarist_cairo_color_t *cairo_palette)
{
    cairo_surface_t *surface;
    atarist_cairo_color_t* dst;
    int rowskip;
    int x;
    int y;
    int i;
#ifdef ATARIST_CAIRO_BITS16
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB16_565,640,400);
#else
    surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,640,400);
#endif
    dst = (atarist_cairo_color_t*)cairo_image_surface_get_data(surface);
    rowskip = cairo_image_surface_get_stride(surface)/sizeof(atarist_cairo_color_t)-640;
    for(y=0;y<400;++y){
        for(x=0;x<40;++x){
            int plane1 = be16toh(atari_image[y*40+x]);
            for(i=15;i>=0;--i)
                *dst++ = plane1 & (1<<i) ? 0 : 0xffffff;
        }
        dst += rowskip;
    }
    cairo_surface_mark_dirty(surface);
    return surface;
}
