/*
 * Copyright (C) 2006-2020  Magnus Bergman
 *
 * Author: Magnus Bergman <magnus.bergman@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*

text attribute bits:

  MSB         LSB
  0 1 2 3 4 5 6 7
  | \___/ | \___/
  |   |   |   |
  |   |   |   `---- text color (EGA style)
  |   |   `-------- text color hilighted
  |   `------------ background color (EGA style)
  `---------------- text blink (or background hilighted with iCE-color)

*/
#ifndef cons_buf_h
#define cons_buf_h

#include "cons_if.h"

#ifndef CONSOLE_BITS_CHAR
# define CONSOLE_BITS_CHAR 8
#endif
#ifndef CONSOLE_BITS_ATTR
# define CONSOLE_BITS_ATTR 8
#endif

typedef struct{
    unsigned int ch:CONSOLE_BITS_CHAR;
    unsigned int attr:CONSOLE_BITS_ATTR;
}ConsoleCell;

Console* cons_buf_new(void);
void cons_buf_free(Console*);
void cons_buf_set_size(Console*,int width,int height,int attr);
void cons_buf_charxya(Console*,int x,int y,int attr,int ch);
//#define text_screen_put(s,x,y,c) (s)->screen[(x)+(y)*(s)->width]=(c)
void conf_buf_scroll(Console*,int steps);
int cons_buf_width(Console*);
int cons_buf_height(Console*);
void cons_buf_extend(Console*,int steps,int attr);
void cons_buf_reshape(Console*,int width);
const ConsoleCell* cons_buf_buffer(Console*);

extern const ConsoleInterface cons_buf_if;

//#define text_screen_get_width_pixels(ts) ((ts)->width*9)
//#define text_screen_get_height_pixels(ts) ((ts)->height*16)

#endif /* cons_buf_h */
