/*
 * Copyright (C) 2018 - 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <nile.h>
#include <nil-cairo.h>
#include <stdlib.h>                     /* malloc(), free() */

#include "ni-koala.h"
#include "nil-c64.h"


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_koala_get_mime_types()
{
    static const char *mime_types[] = {
        "image/x-koa",
        "image/x-koala",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_koala_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h;
    h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    h->info = info;
    info->threadsafe = 1;
    h->surface = NULL;
    return h;
}


static void
_koala_free(abydos_plugin_handle_t *h)
{
    if(h->surface)
        cairo_surface_destroy(h->surface);
    free(h);
}


static int
_koala_create_from_data(abydos_plugin_handle_t *h,const char *data,size_t len)
{
    nile_t ni;
    int res = -1;
    nile_init(&ni);
    nil_c64_palette(&ni, NIL_C64_COLODORE);
    //nil_cairo_convert_palette(&ni);
    if(ni_koala_decode(&ni,(uint8_t*)data,len)<0)
        goto out;
    h->surface = nil_cairo_surface_create(&ni, 0);
    if(!h->surface)
        goto out;
    h->info->width = ni.width;
    h->info->height = ni.height;
    h->info->pixel_ratio = 5.0 / 3;
    res = 0;
out:
    nile_done(&ni);
    return res;
}


static cairo_surface_t *
_koala_get_image_surface(abydos_plugin_handle_t *h,int page,cairo_bool_t *layer_visibility,int variant,double time,int frame)
{
    return cairo_surface_reference(h->surface);
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _koala_get_mime_types,
        .new = _koala_new,
        .free = _koala_free,
        .create_from_data = _koala_create_from_data,
        .get_image_surface = _koala_get_image_surface
    };
    return &p;
}
