/*
 * Copyright (C) 2000 – 2020  LibreOffice contributors
 * Copyright (C) 2021         Magnus Bergman <magnus@snisurset.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#ifndef chart_h
#define chart_h

#include <loa-array.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

//#include "vector.h"

/* FILE TYPE CONSTANTS: */
#define BULCHART        32      /* Bullet chart file.       */
/* the following were added although SPC doesn't have a #define */
/* for them...                                                  */
#define AUTOTTLCHT      95      /* Autobuild TTL CHT        */
#define AUTOBULCHT      96      /* Autobuild BUL CHT        */
#define AUTOTABCHT      97      /* Autobuild TAB CHT        */

typedef struct _TextEntry
{
    uint16_t          nTypeOfText;
    uint16_t          nRowOrLineNum;
    uint16_t          nColumnNum;
    uint16_t          nZoneSize;          // textzone attributes
    uint16_t          nLineType;
    uint16_t          nAttributes;
    char*           pText;              // null terminated text
} TextEntry;

#define text_entry_done(e) if((e)->pText) free((e)->pText)


typedef struct
{
    int16_t           nBoxX1;
    int16_t           nBoxY1;
    int16_t           nBoxX2;
    int16_t           nBoxY2;
    int8_t            nZoneEnum;
}DataNode;

#define data_node_init(n) memset(n, 0, sizeof(DataNode))

typedef struct
{
    int8_t                mnCurrentFileType;
    //::std::vector< std::unique_ptr<TextEntry> > maTextEntryList;
    loa_array_t text_entry_list;
    DataNode                mDataNode[ 7 ];


}CGMChart;

CGMChart *cgm_chart_init(CGMChart*);
void cgm_chart_done(CGMChart*);

void CGMChart_InsertTextEntry(CGMChart*,TextEntry*);
void CGMChart_ResetAnnotation(CGMChart*);
bool CGMChart_IsAnnotation(CGMChart*);

#endif /* chart_h */
