/*
 * Copyright (C) 2000 – 2020  LibreOffice contributors
 * Copyright (C) 2021         Magnus Bergman <magnus@snisurset.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#include "cgmparse.h"
#include "elements.h"


int handle_class0(cgm_context_t *c, int id, cgm_read_t *r)
{
    switch(id){
    case 0x01 : /*Begin Metafile*/
        if(c->state != METAFILE_CLOSED)
            return -1;
        c->state = METAFILE_DESCRIPTION;
        //cgm_set_map_mode(c, r);
        c->mbMetaFile = true;
        break;
    case 0x02 : /*End MetaFile*/
#if 0
        if ( mpBitmapInUse ){
            // process existing graphic
            CGMBitmapDescriptor* pBmpDesc = mpBitmapInUse->GetBitmap();
            // do anything with the bitmap
            /* FIXME!! */
            //vt->DrawBitmap(userdata, pBmpDesc );
            mpBitmapInUse.reset();
        }
#endif
        c->state = METAFILE_CLOSED;
        c->mbIsFinished = true;
        c->mbPictureBody = false;
        c->mbMetaFile = false;
        break;
     case 0x03 : /*Begin Picture*/
        /* FIXME!! */
        //ImplDefaultReplacement();
        cgm_set_map_mode(c, r);
        if(c->mbPicture)
            return -1;

        c->state = PICTURE_DESCRIPTION;
        /* FIXME!! */
        //*pCopyOfE = c.pElement;
        c->mbPicture = c->mbFirstOutPut = true;
        c->mbFigure = false;
        c->mnAct4PostReset = 0;
        if ( 1 /* mpChart == nullptr */ ){
            /* normal CGM Files determines "BeginPic" */
            c->vt->InsertPage(c->userdata, &c->pElement.attr);     // as the next slide
        }
        break;
    case 0x04 : /*Begin Picture Body*/
        if(c->state != PICTURE_DESCRIPTION)
            return -1;
        c->mbPictureBody = true;
        break;
    case 0x05 : /*  End Picture*/
        if(!c->mbPicture)
            return -1;
#if 0
        if ( mpBitmapInUse )                            // process existing graphic
        {
            CGMBitmapDescriptor* pBmpDesc = mpBitmapInUse->GetBitmap();
            // do anything with the bitmap
            /* FIXME!! */
            //vt->DrawBitmap(userdata, pBmpDesc );
            mpBitmapInUse.reset();
        }
#endif
        c->state = METAFILE_DESCRIPTION;
        c->vt->EndFigure(c->userdata);                          // close potential figures
        c->vt->EndGrouping(c->userdata);                        // finish potential groups
        /* FIXME!! */
        //c.pElement = *pCopyOfE;
        c->mbFigure = c->mbFirstOutPut = c->mbPicture = c->mbPictureBody = false;
        break;
    case 0x06 : /*Begin Segment*/
        c->pElement.bSegmentCount = true;
        break;
    case 0x07 : /*End Segment*/
        c->pElement.bSegmentCount = true;
        break;
    case 0x08 : /*Begin Figure*/
        c->mbFigure = true;
        c->vt->BeginFigure(c->userdata);
        break;
    case 0x09 : /*End Figure*/
        c->vt->EndFigure(c->userdata);
        c->mbFigure = false;
        break;
    case 0x0d : /*Begin Protection vcl::Region */
        break;
    case 0x0e : /*End Protection vcl::Region */
        break;
    case 0x0f : /*Begin Compound Line */
        break;
    case 0x10 : /*End Compound Line */
        break;
    case 0x11 : /*Begin Compound Text Path */
        break;
    case 0x12 : /*End Compound Text Path */
        break;
    case 0x13 : /*Begin Tile Array */
        break;                 // NS
    case 0x14 : /*End Tile Array */
        break;                   // NS
    case 0xff : /*Filter Setup */
        break;
    case 0xfe : /*Begin Block Text vcl::Region */
        break;
    case 0xfd : /*End Block Text vcl::Region */
        break;
    case 0xfc : /*Begin Group*/
        c->vt->BeginGroup(c->userdata);
        break;
    case 0xfb : /*End Group*/
        c->vt->EndGroup(c->userdata);
        break;
    case 0xfa : /*Begin Patch */
        break;
    case 0xf9 : /*Begin Patch */
        break;
    default:
        break;
    }
    return 0;
};
