/*
 * Copyright (C) 2000 – 2020  LibreOffice contributors
 * Copyright (C) 2021         Magnus Bergman <magnus@snisurset.net>
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#include "cgmparse.h"
#include "elements.h"


int handle_class3(cgm_context_t *c, int id, cgm_read_t *r)
{
    switch(id){
    case 0x01 : /*VDC Integer Precision*/
        r->vdc_int_precision = r_get_int_precision_bytes(r);
        break;
    case 0x02 : /*VDC Real Precision*/
        {
            vdc_type_t type = r_get_vdc_type(r);
            const long nI0 = r_get_I(r); /* exponent */
            const long nI1 = r_get_I(r); /* mantissa */
            switch( type )
            {
                case 0 :
                    r->vdc_real_type = RT_FLOAT;
                    switch ( nI0 )
                    {
                        case 9 :
                            if ( nI1 != 23 )
                                return -1;
                            r->vdc_real_size = 4;
                            break;
                        case 12 :
                            if ( nI1 != 52 )
                                return -1;
                            r->vdc_real_size = 8;
                            break;
                        default:
                            return -1;
                    }
                    break;
                case 1 :
                    r->vdc_real_type = RT_FIXED;
                    if ( nI0 != nI1 )
                        return -1;
                    if ( nI0 == 16 )
                        r->vdc_real_size = 4;
                    else if ( nI0 == 32 )
                        r->vdc_real_size = 8;
                    else
                        return -1;
                    break;
                default :
                    return -1;
            }
        }
        break;
    case 0x03 : /* Auxiliary Colour (CO) */
        c->pElement.nAuxiliaryColor = r_get_color(r, &c->pElement, 0);
        break;
    case 0x04 : /* Transparency (E) */
        c->pElement.eTransparency = r_get_transparency(r);
        break;
    case 0x05 : /* Clip Rectangle (P, P) */
        r_get_rect(r, &c->pElement.aClipRect );
        break;
    case 0x06 : /* Clip Indicator (E) */
        c->pElement.eClipIndicator = r_get_clip_indicator(r);
        break;
    case 0x07 : /* Line Clipping Mode (E) */
        break;               // NS
    case 0x08 : /* Marker Clipping Mode (E) */
        break;             // NS
    case 0x09 : /* Edge Clipping Mode (E) */
        break;               // NS
    case 0x0a : /* New Region */
        c->vt->NewRegion(c->userdata);
        break;
    case 0x0b : /* Save Primitive Context (N) */
        break;           // NS
    case 0x0c : /* Restore Primitive Context */
        break;        // NS
    case 0x11 : /* Protection Region Indicator (IX, IX) */
        break;
    case 0x12 : /* Generalized Text Path Mode (E) */
        break;       // NS
    case 0x13 : /* Mitre Limit (R) */
        c->pElement.nMitreLimit = r_get_R(r);
        break;                                                                      // NS
    case 0x14 : /* Transparent Cell Color (E, CO) */
        break;           // NS
    case 0xfc : /* Text Path Alignment Modes */
        break;
    case 0xfd : /* Pop Transformation Stack */
        break;
    case 0xfe : /* Push Transformation Stack */
        break;
    case 0xff : /*Set Patch ID */
        break;
    default:
        break;
    }
    return 0;
};
