/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <nile.h>
#include <nil-cairo.h>
#include <stdlib.h>
#include <string.h>

#include "nil-fax.h"


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_g3_get_mime_types()
{
    static const char *mime_types[] = {
        "image/g3fax",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_g3_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    info->threadsafe = 1;
    h->info = info;
    h->surface = NULL;
    return h;
}


static void
_g3_free(abydos_plugin_handle_t *h)
{
    if(h->surface)
        cairo_surface_destroy(h->surface);
    free(h);
}


static int
_g3_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    static const uint8_t palette[6] = {0,0,0,255,255,255};
    nile_t ni;
    nile_init(&ni);
    ni.width = 1728;
    ni.height = 1728;
    nile_ensure_indexed(&ni, ni.width, ni.height);
    ni.palette = (uint8_t*)palette;
    ni.colors = 2;
    ni.bpc = 3;
    ni.format = NILE_FORMAT_r8g8b8;
    nil_fax_decode(&ni, NIL_FAX_MH, (uint8_t*)data, len);
    h->surface = nil_cairo_surface_create(&ni, 0);
    nile_done(&ni);
    return 0;
}


static cairo_surface_t *
_g3_get_image_surface(abydos_plugin_handle_t *h,
                      int page,
                      cairo_bool_t *layer_visibility,
                      int variant,
                      double time,
                      int frame)
{
    return cairo_surface_reference(h->surface);
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _g3_get_mime_types,
        .new = _g3_new,
        .free = _g3_free,
        .create_from_data = _g3_create_from_data,
        .get_image_surface = _g3_get_image_surface
    };
    return &p;
}
