/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <debug.h>
#include <string.h>                     /* memcmp() */

#include "ni-xng.h"


#define get32(a) be32toh(*(uint32_t*)(a))


int
ni_xng_parse(const char *signature, uint32_t endid, const char *data, size_t len, int validate, ni_xng_parse_func_t parse, void *userdata)
{
    const char *end = data + len;

    if(signature){
        if(len < 8)
            return -1;
        DEBUG_p("signature: %x:%x", get32(data), get32(data+4));
        if(memcmp(data, signature, 8)){
            DEBUG_p("wrong signature");
            return -1;
        }
        data += 8;
    }

    for(;;){
        uint32_t chunk_len;
        uint32_t chunk_id;
        int res;
        if(data + 12 > end){
            if(endid){
                DEBUG_p("reached the end without finding the end chunk");
                return -1;
            }
            return 0;
        }
        chunk_len = get32(data);
        chunk_id = *(uint32_t*)(data+4);

        DEBUG_p("chunk id: %.4s, length: %d", data+4, chunk_len);
        data += 8;

        if(data + chunk_len + 4 > end){
            DEBUG_p("end in the middle of a chunk");
            return -1;
        }

        res = parse(userdata, chunk_id, data, chunk_len);
        if(res){
            DEBUG_p("parse error");
            return res;
        }

        if(validate){
            /* FIXME!! calculate and compare checksum. */
        }

        if(endid && endid == chunk_id)
            return 0;

        data += chunk_len + 4; /* also skip past checksum */
    }
}
