/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef xmlutil_h
#define xmlutil_h

#include <loa-buffer.h>

typedef struct{
    double r;
    double g;
    double b;
    double a;
}color_t;

typedef struct _xml_handle_t xml_handle_t;
typedef struct _xml_attributes_t xml_attributes_t;

typedef void(*xml_begin_func_t)(void *userdata, const char *element, xml_attributes_t *attr);
typedef void(*xml_end_func_t)(void *userdata, const char *element);

#define SKIP_SPACE(s) for(;' '==*s || ','==*(s);++(s))
#define SKIP_SEP(s) for(;' '==*s || ','==*(s) || ';'==*(s);++(s))

int _get_int(const char **s);
double _get_double(const char **s);
void _get_point(loa_point_t *p, const char **s, loa_point_t *last, int op);

void _color_set_hexstring(color_t *c, const char *s);

xml_handle_t *xml_parser_begin(xml_begin_func_t begin, xml_end_func_t end, void *userdata);
int xml_parser_feed(xml_handle_t *handle, const char *data, ssize_t len);
int xml_parser_end(xml_handle_t *handle);
const char *xml_parser_attr_string(const xml_attributes_t *attr, const char *name);
int xml_parser_attr_int(const xml_attributes_t *attr, const char *name, int fallback);
double xml_parser_attr_double(const xml_attributes_t *attr, const char *name, int fallback);

#endif /* xmlutil_h */
