/*
 * Copyright (C) 2017  Teemu Suutari
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 */
#ifndef OUTPUTSTREAM_HPP
#define OUTPUTSTREAM_HPP

#include <cstddef>
#include <cstdint>

#include "buffer.h"
#include "stream.h"

class OutputStream : public Stream {
protected:
    OutputStream(uint8_t *bufPtr, size_t startOffset,size_t currentOffset,size_t endOffset);
    OutputStream(Buffer &buffer,size_t startOffset,size_t currentOffset,size_t endOffset);
    ~OutputStream(){}

    uint8_t *_bufPtr;
};

class ForwardOutputStream : public OutputStream {
public:
    ForwardOutputStream(uint8_t *bufPtr,size_t startOffset,size_t endOffset) : OutputStream(bufPtr, startOffset, startOffset, endOffset) {}
    ForwardOutputStream(Buffer &buffer,size_t startOffset,size_t endOffset) : OutputStream(buffer, startOffset, startOffset, endOffset) {}
    ~ForwardOutputStream(){}

    void writeByte(uint8_t value);

    uint8_t copy(size_t distance,size_t count);
    uint8_t copy(size_t distance,size_t count,const Buffer &prevBuffer);
    uint8_t copy(size_t distance,size_t count,uint8_t defaultChar);
    const uint8_t *history(size_t distance) const;
    void produce(const uint8_t *src,size_t bytes);

    bool eof() const { return _currentOffset==_endOffset; }
    size_t getEndOffset() const { return _endOffset; }
};

class BackwardOutputStream : public OutputStream {
public:
    BackwardOutputStream(Buffer &buffer,size_t startOffset,size_t endOffset) : OutputStream(buffer, startOffset, endOffset, endOffset) {}
    ~BackwardOutputStream(){}

    void writeByte(uint8_t value);

    uint8_t copy(size_t distance,size_t count);

    bool eof() const { return _currentOffset==_startOffset; }
};

#endif
