/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef list_h
#define list_h

#ifdef USE_GLIB
//# define LIST_USE_GLIB
#endif

#ifdef LIST_USE_MOWGLI

#define List mowgli_node_t

#elif defined(LIST_USE_GLIB)

#include <glib.h>

#define List GList
#define ListNotifyFunc GDestroyNotify
#define ListCompFunc GCompareFunc
#define ListForeachFunc GFunc
#define list_new                g_list_new
#define list_free               g_list_free
#define list_free_full          g_list_free_full
#define list_insert_before      g_list_insert_before
#define list_insert_after       g_list_insert_after
#define list_insert_sorted      g_list_insert_sorted
#define list_prepend            g_list_prepend
#define list_append             g_list_append
#define list_find_custom(a,b,c) g_list_find_custom(a,c,b)
#define list_reverse(a)         g_list_reverse(a)
#define list_foreach(a,b,c)     g_list_foreach(a,b,c)
#define list_remove             g_list_delete_link
#define list_remove_data        g_list_remove

#else

typedef struct _List List;
struct _List{
    void* data;
    List* next;
    List* prev;
};

typedef void(*ListNotifyFunc)(void*);
typedef int(*ListCompFunc)(const void*,const void*);
typedef void(*ListForeachFunc)(void*,void*);

#define list_new() NULL
void    list_free(List*);
void    list_free_full(List*,ListNotifyFunc);
List*   list_insert_before(List* list,List* node,void* data);
List*   list_insert_after(List* list,List* node,void* data);
List*   list_insert_sorted(List*,void* data,ListCompFunc);
#define list_prepend(list,data) list_insert_before(list,NULL,data)
#define list_append(list,data) list_insert_after(list,NULL,data)
List*   list_find_custom(List*,ListCompFunc,void*);
List*   list_find_data(List*,void*);
void    list_foreach(List*,ListForeachFunc,void*);
List*   list_remove(List* list,List* node);
List*   list_remove_custom(List* list,ListCompFunc,void* data);
List*   list_remove_data(List* list,const void* data);

#endif /* LIST_USE_GLIB */

#endif /* list_h */
