/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <stdlib.h>                     /* malloc() */
#include <string.h>                     /* memcpy() */

#include "loa-array.h"


#define MIN_SIZE 16

loa_array_t *
loa_array_init(loa_array_t *a, int membersize)
{
    if((a->membersize = membersize)){
        a->allocation = a->membersize * MIN_SIZE;
        a->data = malloc(a->allocation);
    }
    a->members = 0;
    return a;
}


void
loa_array_done(loa_array_t *a)
{
    if(a->membersize)
        free(a->data);
}


void
loa_array_ensure(loa_array_t *a, int count)
{
    int newsize = count * a->membersize;
    if(newsize > a->allocation){
        while(newsize > a->allocation)
            a->allocation *= 2;
        a->data = realloc(a->data, a->allocation);
    }
}


void *
loa_array_append(loa_array_t *a, int count){
    int index = a->members;
    a->members += count;
    loa_array_ensure(a, a->members);
    return a->data + index*a->membersize;
}


void *
loa_array_insert(loa_array_t *a, int pos, int count){
    int move_members = a->members - pos;
    void *p = loa_array_append(a,count);
    if(move_members > 0){
        p = a->data + pos*a->membersize;
        memmove(p + count*a->membersize, p, move_members*a->membersize);
    }
    return p;
}


void
loa_array_delete(loa_array_t *a, int pos, int count){
    int move_members = a->members - (pos+count);
    if(move_members > 0)
        memmove(a->data + pos*a->membersize,
                a->data + (pos+count)*a->membersize,
                move_members*a->membersize);
    a->members -= count;
}


void *
loa_array_copy(loa_array_t *a)
{
    size_t size = loa_array_bytes(a);
    void *copy = malloc(size);
    memcpy(copy, a->data, size);
    return copy;
}
